@@ -1,7 +1,12 @@
 #' @import data.table
 #' @import Matrix
-#' @importFrom stats fisher.test quantile xtabs
+#' @import shiny
+#' @import plotly
+#' @import lattice
+#' @importFrom stats fisher.test quantile xtabs qbeta
 #' @importFrom piggyback pb_download
+#' @importFrom grid grid.polygon gpar
+
 
 
 
@@ -14,5 +19,7 @@ IBD <- function(ibd_data = "name.ibd", caco = "name.Rda", ...) {
   return(ibd)
 }
 
+utils::globalVariables(c("caco_file_name", "data_folder_address"))
+
 
 
